@@ -5,8 +5,6 @@
 # Rene Mayrhofer, July 2000
 # Some code has been taken from redhat-linux-lib.pl
 
-use File::Copy;
-
 $network_interfaces_config = '/etc/network/interfaces';
 $modules_config = '/etc/modprobe.d/arch/i386';
 if (!-d $modules_config) {
@@ -387,7 +385,7 @@ sub modify_module_def
 	my $modify_block = 0;
 	
 	# make a backup copy
-	copy("$modules_config", "$modules_config~");
+	&copy_source_dest($modules_config, "$modules_config~");
 	local *OLDCFGFILE, *NEWCFGFILE;
 	&open_readfile(OLDCFGFILE, "$modules_config~") ||
 		error("Unable to open $modules_config");
@@ -489,7 +487,7 @@ sub new_module_def
 {
 	local ($name, $mode, $miimon, $downdelay, $updelay) = @_;
         return if (!$modules_config);
-	copy("$modules_config", "$modules_config~");
+	&copy_source_dest($modules_config, "$modules_config~");
 	local *CFGFILE;
 	&open_lock_tempfile(CFGFILE, ">> $modules_config") ||
 		error("Unable to open $modules_config");
@@ -549,6 +547,7 @@ return &check_ipaddress_any($_[0]);
 }
 
 # get_hostname()
+# Returns the saved hostname
 sub get_hostname
 {
 local $hn = &read_file_contents("/etc/hostname");
@@ -560,14 +559,15 @@ return &get_system_hostname();
 }
 
 # save_hostname(name)
+# Update the system's saved and live hostname
 sub save_hostname
 {
-local (%conf, $f);
-&system_logged("hostname $_[0] >/dev/null 2>&1");
-foreach $f ("/etc/hostname", "/etc/HOSTNAME", "/etc/mailname") {
+my ($hostname) = @_;
+&system_logged("hostname ".quotemeta($hostname)." >/dev/null 2>&1");
+foreach my $f ("/etc/hostname", "/etc/HOSTNAME", "/etc/mailname") {
 	if (-r $f) {
 		&open_lock_tempfile(HOST, ">$f");
-		&print_tempfile(HOST, $_[0],"\n");
+		&print_tempfile(HOST, $hostname,"\n");
 		&close_tempfile(HOST);
 		}
 	}
@@ -586,8 +586,8 @@ return $d;
 # save_domainname(domain)
 sub save_domainname
 {
-local %conf;
-&execute_command("domainname ".quotemeta($_[0]));
+my ($domain) = @_;
+&execute_command("domainname ".quotemeta($domain));
 }
 
 sub routing_config_files
@@ -779,8 +779,7 @@ sub get_interface_defs
 {
 local *CFGFILE;
 my @ret;
-&open_readfile(CFGFILE, $network_interfaces_config) ||
-	error("Unable to open $network_interfaces_config");
+&open_readfile(CFGFILE, $network_interfaces_config) || return ();
 # read the file line by line
 $line = <CFGFILE>;
 while (defined $line) {
@@ -902,7 +901,7 @@ sub modify_interface_def
 {
 my ($name, $addrfam, $method, $options, $mode) = @_;
 # make a backup copy
-copy("$network_interfaces_config", "$network_interfaces_config~");
+&copy_source_dest($network_interfaces_config, "$network_interfaces_config~");
 local *OLDCFGFILE, *NEWCFGFILE;
 &open_readfile(OLDCFGFILE, "$network_interfaces_config~") ||
 	error("Unable to open $network_interfaces_config");
@@ -969,7 +968,7 @@ close(OLDCFGFILE);
 sub new_interface_def
 {
 # make a backup copy
-copy("$network_interfaces_config", "$network_interfaces_config~");
+&copy_source_dest($network_interfaces_config, "$network_interfaces_config~");
 local *CFGFILE;
 &open_lock_tempfile(CFGFILE, ">> $network_interfaces_config") ||
 	error("Unable to open $network_interfaces_config");
