@@ -116,6 +116,13 @@ foreach my $f (glob("$netplan_dir/*.yaml")) {
 			$cfg->{'ether'} = $macaddress->{'value'};
 			}
 
+		# Static routes
+		my ($routes) = grep { $_->{'name'} eq 'routes' }
+                                    @{$e->{'members'}};
+		if ($routes) {
+			$cfg->{'routes'} = $routes;
+			}
+
 		# Add IPv4 alias interfaces
 		my $i = 0;
 		foreach my $aa (@addrs) {
@@ -210,10 +217,13 @@ else {
 	if ($iface->{'ether'}) {
 		push(@lines, $id."    "."macaddress: ".$iface->{'ether'});
 		}
+	if ($iface->{'routes'}) {
+		push(@lines, &yaml_lines($iface->{'routes'}, $id."    "));
+		}
 
 	# Add all extra YAML directives from the original config
-	my @poss = ( "optional", "dhcp4", "dhcp6", "addresses", "gateway4",
-		     "gateway6", "nameservers", "macaddress" );
+	my @poss = ("optional", "dhcp4", "dhcp6", "addresses", "gateway4",
+		    "gateway6", "nameservers", "macaddress", "routes");
 	if ($iface->{'yaml'}) {
 		foreach my $y (@{$iface->{'yaml'}->{'members'}}) {
 			next if (&indexof($y->{'name'}, @poss) >= 0);
@@ -325,7 +335,7 @@ return $_[0] =~ /^(eth|em)/;
 # Returns 1 if managing IPv6 interfaces is supported
 sub supports_address6
 {
-local ($iface) = @_;
+my ($iface) = @_;
 return !$iface || $iface->{'virtual'} eq '';
 }
 
@@ -365,7 +375,7 @@ return &check_ipaddress_any($_[0]);
 # get_hostname()
 sub get_hostname
 {
-local $hn = &read_file_contents("/etc/hostname");
+my $hn = &read_file_contents("/etc/hostname");
 $hn =~ s/\r|\n//g;
 if ($hn) {
 	return $hn;
@@ -376,8 +386,9 @@ return &get_system_hostname();
 # save_hostname(name)
 sub save_hostname
 {
-local (%conf, $f);
-&system_logged("hostname $_[0] >/dev/null 2>&1");
+my ($hostname) = @_;
+my (%conf, $f);
+&system_logged("hostname ".quotemeta($hostname)." >/dev/null 2>&1");
 foreach $f ("/etc/hostname", "/etc/HOSTNAME", "/etc/mailname") {
 	if (-r $f) {
 		&open_lock_tempfile(HOST, ">$f");
@@ -391,7 +402,7 @@ undef(@main::get_system_hostname);      # clear cache
 # get_domainname()
 sub get_domainname
 {
-local $d;
+my $d;
 &execute_command("domainname", undef, \$d, undef);
 chop($d);
 return $d;
@@ -400,8 +411,8 @@ return $d;
 # save_domainname(domain)
 sub save_domainname
 {
-local %conf;
-&execute_command("domainname ".quotemeta($_[0]));
+my ($domain) = @_;
+&execute_command("domainname ".quotemeta($domain));
 }
 
 sub routing_config_files
@@ -416,6 +427,7 @@ sub routing_input
 my ($addr, $router) = &get_default_gateway();
 my ($addr6, $router6) = &get_default_ipv6_gateway();
 my @ifaces = grep { $_->{'virtual'} eq '' } &boot_interfaces();
+my @inames = map { $_->{'name'} } @ifaces;
 
 # Show default gateway
 print &ui_table_row($text{'routes_default'},
@@ -423,8 +435,7 @@ print &ui_table_row($text{'routes_default'},
 		  [ [ 1, $text{'routes_none'} ],
 		    [ 0, $text{'routes_gateway'}." ".
 			 &ui_textbox("gateway", $addr, 15)." ".
-			 &ui_select("gatewaydev", $router,
-				[ map { $_->{'name'} } @ifaces ]) ] ]));
+			 &ui_select("gatewaydev", $router, \@inames) ] ]));
 
 # Show default IPv6 gateway
 print &ui_table_row($text{'routes_default6'},
@@ -432,15 +443,44 @@ print &ui_table_row($text{'routes_default6'},
 		  [ [ 1, $text{'routes_none'} ],
 		    [ 0, $text{'routes_gateway'}." ".
 			 &ui_textbox("gateway6", $addr6, 30)." ".
-			 &ui_select("gatewaydev6", $router6,
-				[ map { $_->{'name'} } @ifaces ]) ] ]));
+			 &ui_select("gatewaydev6", $router6, \@inames) ] ]));
 
 # Act as router?
-local %sysctl;
+my %sysctl;
 &read_env_file($sysctl_config, \%sysctl);
 print &ui_table_row($text{'routes_forward'},
 	&ui_yesno_radio("forward",
 			$sysctl{'net.ipv4.ip_forward'} ? 1 : 0));
+
+# Static routes
+my $rtable = &ui_columns_start([ $text{'routes_ifc'},
+			       $text{'routes_net'},
+			       $text{'routes_mask'},
+			       $text{'routes_gateway'} ]);
+my $i = 0;
+@inames = ( "", @inames );
+foreach my $b (@ifaces) {
+	next if (!$b->{'routes'});
+	foreach my $v (@{$b->{'routes'}->{'value'}}) {
+		my ($net, $mask) = split(/\//, $v->{'to'});
+		$mask = &prefix_to_mask($mask);
+		$rtable .= &ui_columns_row([
+		    &ui_select("dev_$i", $b->{'fullname'}, \@inames, 1, 0, 1),
+		    &ui_textbox("net_$i", $net, 15),
+		    &ui_textbox("mask_$i", $mask, 15),
+		    &ui_textbox("gw_$i", $v->{'via'}, 15),
+		    ]);
+		$i++;
+		}
+	}
+$rtable .= &ui_columns_row([
+	&ui_select("dev_$i", "", \@inames, 1, 0, 1),
+	&ui_textbox("net_$i", "", 15),
+	&ui_textbox("mask_$i", "", 15),
+	&ui_textbox("gw_$i", "", 15),
+	]);
+$rtable .= &ui_columns_end();
+print &ui_table_row($text{'routes_static'}, $rtable);
 }
 
 # parse_routing()
@@ -461,19 +501,46 @@ if (!$in{'gateway_def'}) {
 my ($dev6, $gw6);
 if (!$in{'gateway6_def'}) {
 	&check_ip6address($in{'gateway6'}) ||
-		&error(&text('routes_egateway6', &html_escape($in{'gateway6'})));
+		&error(&text('routes_egateway6',&html_escape($in{'gateway6'})));
 	$gw6 = $in{'gateway6'};
 	$dev6 = $in{'gatewaydev6'};
 	}
 &set_default_ipv6_gateway($gw6, $dev6);
 
 # Save routing flag
-local %sysctl;
+my %sysctl;
 &lock_file($sysctl_config);
 &read_env_file($sysctl_config, \%sysctl);
 $sysctl{'net.ipv4.ip_forward'} = $in{'forward'};
 &write_env_file($sysctl_config, \%sysctl);
 &unlock_file($sysctl_config);
+
+# Save static routes
+my @boot =  &boot_interfaces();
+foreach my $b (grep { $_->{'virtual'} eq '' } @boot) {
+	my @r;
+	for(my $i=0; defined($in{"dev_$i"}); $i++) {
+		if ($in{"dev_$i"} eq $b->{'fullname'}) {
+			&check_ipaddress($in{"net_$i"}) ||
+				&error(&text('routes_enet', $in{"net_$i"}));
+			&check_ipaddress($in{"mask_$i"}) ||
+				&error(&text('routes_emask', $in{"mask_$i"}));
+			my $to = $in{"net_$i"}."/".
+				 &mask_to_prefix($in{"mask_$i"});
+			&check_ipaddress($in{"gw_$i"}) ||
+				&error(&text('routes_egateway', $in{"gw_$i"}));
+			push(@r, { 'to' => $to, 'via' => $in{"gw_$i"} });
+			}
+		}
+	if (@r) {
+		$b->{'routes'} = { 'name' => 'routes',
+				   'value' => \@r };
+		}
+	else {
+		delete($b->{'routes'});
+		}
+	&save_interface($b, \@boot);
+	}
 }
 
 sub network_config_files
@@ -634,7 +701,7 @@ foreach my $origl (@$lref) {
 			&set_parent_elines($parent, $lnum);
 			}
 		}
-	elsif ($l =~ /^(\s*)\-\s*(\S+):\s*(\S.*)$/) {
+	elsif ($l =~ /^(\s*)\-\s*(\S+):\s+(\S.*)$/) {
 		# Value that is itself a key-value pair
 		# routes:
 		#   - to: 1.2.3.4/24
