@@ -61,12 +61,12 @@ do 'linux-lib.pl';
 # Returns a list of interfaces brought up at boot time
 sub boot_interfaces
 {
-local (@rv, $f);
-local %bridge_map;
-local @active;
+my (@rv, $f);
+my %bridge_map;
+my @active;
 opendir(CONF, &translate_filename($net_scripts_dir));
 while($f = readdir(CONF)) {
-	local (%conf, $b);
+	my (%conf, $b);
 	if ($f =~ /^ifcfg-([a-z0-9:\.]+)\-range([a-z0-9\.\_]+)$/) {
 		# A range of addresses
 		&read_env_file("$net_scripts_dir/$f", \%conf);
@@ -139,10 +139,6 @@ while($f = readdir(CONF)) {
 			$b->{'netmask'} = &prefix_to_mask($conf{'PREFIX0'});
 			}
 		$b->{'broadcast'} = $conf{'BROADCAST'};
-		if (!$b->{'broadcast'} && $b->{'address'} && $b->{'netmask'}) {
-			$b->{'broadcast'} = &compute_broadcast($b->{'address'},
-							       $b->{'netmask'});
-			}
 		$b->{'gateway'} = $conf{'GATEWAY'};
 		$b->{'gateway6'} = $conf{'IPV6_DEFAULTGW'};
 		$b->{'mtu'} = $conf{'MTU'};
@@ -168,7 +164,7 @@ while($f = readdir(CONF)) {
 		$b->{'ether'} = $conf{'MACADDR'};
 		$b->{'dhcp'} = ($conf{'BOOTPROTO'} eq 'dhcp');
 		$b->{'bootp'} = ($conf{'BOOTPROTO'} eq 'bootp');
-		local @ip6s;
+		my @ip6s;
 		push(@ip6s, [ split(/\//, $conf{'IPV6ADDR'}) ])
 			if ($conf{'IPV6ADDR'});
 		push(@ip6s, map { [ split(/\//, $_) ] }
@@ -189,6 +185,25 @@ while($f = readdir(CONF)) {
 			$bridge_map{$conf{'BRIDGE'}} = $b->{'fullname'};
 			}
 		push(@rv, $b);
+
+		# Extra IPADDRn lines in the config are virtual IPs
+		my $i = 0;
+		while(defined($conf{'IPADDR'.($i+1)})) {
+			my $acfg = { 'name' => $b->{'name'},
+				     'virtual' => $i,
+				     'fullname' => $b->{'name'}.":".$i,
+				     'file' => $b->{'file'},
+				     'parent' => $b->{'fullname'},
+				     'edit' => 1,
+				     'up' => 1, };
+			$acfg->{'address'} = $conf{'IPADDR'.($i+1)};
+			$acfg->{'netmask'} = $conf{'NETMASK'.($i+1)} ||
+				&prefix_to_mask($conf{'PREFIX'.($i+1)}) ||
+				$b->{'netmask'};
+			$acfg->{'index'} = scalar(@rv);
+			push(@rv, $acfg);
+			$i++;
+			}
 		}
 	}
 closedir(CONF);
@@ -222,144 +237,163 @@ $conf{'USERCTL'} = "no";
 sub save_interface
 {
 my ($b) = @_;
-local(%conf);
-local $name = $b->{'range'} ne "" ? $b->{'name'}."-range".$b->{'range'} :
-	      $b->{'virtual'} ne "" ? $b->{'name'}.":".$b->{'virtual'} :
-	      $b->{'vlanid'} ne "" ? $b->{'physical'}.".".$b->{'vlanid'}
-				       : $b->{'name'};
-my $file = $b->{'file'} || "$net_scripts_dir/ifcfg-$name";
-&lock_file($file);
-&read_env_file($file, \%conf);
-if ($b->{'range'} ne "") {
-	# Special case - saving a range
-	$conf{'IPADDR_START'} = $b->{'start'};
-	$conf{'IPADDR_END'} = $b->{'end'};
-	$conf{'CLONENUM_START'} = $b->{'num'};
+my $pmode = $b->{'parent'} && $b->{'virtual'} ne '' ||
+	    $gconfig{'os_version'} >= 16 && $b->{'virtual'} ne '';
+my $name = $b->{'range'} ne "" ? $b->{'name'}."-range".$b->{'range'} :
+	   $b->{'virtual'} ne "" ? $b->{'name'}.":".$b->{'virtual'} :
+	   $b->{'vlanid'} ne "" ? $b->{'physical'}.".".$b->{'vlanid'}
+				: $b->{'name'};
+if ($pmode) {
+	# Virtual interface being updated in it's parent's config file
+	$name =~ s/:\d+$//;
+	my $file = $b->{'file'} || "$net_scripts_dir/ifcfg-$name";
+	&lock_file($file);
+	&read_env_file($file, \%conf);
+	my $n = $b->{'virtual'} + 1;
+	$conf{'IPADDR'.$n} = $b->{'address'};
+	$conf{'PREFIX'.$n} = &mask_to_prefix($b->{'netmask'});
+	delete($conf{'NETMASK'.$n});
+	&write_env_file($file, \%conf);
+	&unlock_file($file);
+	$b->{'parent'} ||= $b->{'name'};
 	}
 else {
-	# Saving a normal interface
-	$conf{'DEVICE'} = $name;
-	my $pfx = $conf{'IPADDR0'} ? '0' : '';
-	$conf{'IPADDR'.$pfx} = $b->{'address'};
-	$conf{'NETMASK'.$pfx} = $b->{'netmask'};
-	delete($conf{'PREFIX'.$pfx});
-	if ($b->{'address'} && $b->{'netmask'}) {
-		$conf{'NETWORK'.$pfx} = &compute_network($b->{'address'},
-						         $b->{'netmask'});
+	# Interface has it's own file
+	local(%conf);
+	my $file = $b->{'file'} || "$net_scripts_dir/ifcfg-$name";
+	&lock_file($file);
+	&read_env_file($file, \%conf);
+	if ($b->{'range'} ne "") {
+		# Special case - saving a range
+		$conf{'IPADDR_START'} = $b->{'start'};
+		$conf{'IPADDR_END'} = $b->{'end'};
+		$conf{'CLONENUM_START'} = $b->{'num'};
 		}
 	else {
-		$conf{'NETWORK'.$pfx} = '';
-		}
-	$conf{'BROADCAST'.$pfx} = $b->{'broadcast'};
-	if ($b->{'gateway'}) {
-		$conf{'GATEWAY'} = $b->{'gateway'};
-		}
-	else {
-		delete($conf{'GATEWAY'});
-		}
-	if ($b->{'gateway6'}) {
-		$conf{'IPV6_DEFAULTGW'} = $b->{'gateway6'};
-		}
-	else {
-		delete($conf{'IPV6_DEFAULTGW'});
-		}
-	$conf{'MTU'} = $b->{'mtu'};
-	$conf{'MACADDR'} = $b->{'ether'};
-	$conf{'ONBOOT'} = $b->{'up'} ? "yes" : "no";
-	$conf{'ONPARENT'} = $b->{'up'} ? "yes" : "no"
-		if ($b->{'virtual'} ne '');
-	$conf{'BOOTPROTO'} = $b->{'bootp'} ? "bootp" :
-			     $b->{'dhcp'} ? "dhcp" : "none";
-	delete($conf{'IPV6ADDR'});
-	delete($conf{'IPV6ADDR_SECONDARIES'});
-	local @ip6s;
-	for(my $i=0; $i<@{$b->{'address6'}}; $i++) {
-		push(@ip6s, $b->{'address6'}->[$i]."/".
-			    $b->{'netmask6'}->[$i]);
-		}
-	if ((@ip6s || $b->{'auto6'}) && lc($conf{'IPV6INIT'}) ne 'yes') {
-		$conf{'IPV6INIT'} = 'yes';
-		}
-	elsif (!@ip6s && !$b->{'auto6'}) {
-		$conf{'IPV6INIT'} = 'no';
-		}
-	if (@ip6s) {
-		$conf{'IPV6ADDR'} = shift(@ip6s);
-		$conf{'IPV6ADDR_SECONDARIES'} = join(" ", @ip6s);
-		}
-	if ($b->{'fullname'} =~ /^br(\d+)$/) {
-		&has_command("brctl") ||
-			&error("Bridges cannot be created unless the brctl ".
-			       "command is installed");
-		$conf{'TYPE'} = 'Bridge';
-		}
-	if ($b->{'fullname'} =~ /^bond(\d+)$/) {
-		$conf{'BONDING_OPTS'} = "mode=$b->{'mode'}";
-		if ($b->{'miimon'}) {
-			$conf{'BONDING_OPTS'} .= " miimon=$b->{'miimon'}";
+		# Saving a normal interface
+		$conf{'DEVICE'} = $name;
+		my $pfx = $conf{'IPADDR0'} ? '0' : '';
+		$conf{'IPADDR'.$pfx} = $b->{'address'};
+		$conf{'NETMASK'.$pfx} = $b->{'netmask'};
+		delete($conf{'PREFIX'.$pfx});
+		if ($b->{'address'} && $b->{'netmask'}) {
+			$conf{'NETWORK'.$pfx} = &compute_network($b->{'address'},
+								 $b->{'netmask'});
 			}
-		if ($b->{'updelay'}) {
-			$conf{'BONDING_OPTS'} .= " updelay=$b->{'updelay'}";
+		else {
+			$conf{'NETWORK'.$pfx} = '';
 			}
-		if ($b->{'downdelay'}) {
-			$conf{'BONDING_OPTS'} .= " downdelay=$b->{'downdelay'}";
+		$conf{'BROADCAST'.$pfx} = $b->{'broadcast'};
+		if ($b->{'gateway'}) {
+			$conf{'GATEWAY'} = $b->{'gateway'};
 			}
+		else {
+			delete($conf{'GATEWAY'});
+			}
+		if ($b->{'gateway6'}) {
+			$conf{'IPV6_DEFAULTGW'} = $b->{'gateway6'};
+			}
+		else {
+			delete($conf{'IPV6_DEFAULTGW'});
+			}
+		$conf{'MTU'} = $b->{'mtu'};
+		$conf{'MACADDR'} = $b->{'ether'};
+		$conf{'ONBOOT'} = $b->{'up'} ? "yes" : "no";
+		$conf{'ONPARENT'} = $b->{'up'} ? "yes" : "no"
+			if ($b->{'virtual'} ne '');
+		$conf{'BOOTPROTO'} = $b->{'bootp'} ? "bootp" :
+				     $b->{'dhcp'} ? "dhcp" : "none";
+		delete($conf{'IPV6ADDR'});
+		delete($conf{'IPV6ADDR_SECONDARIES'});
+		my @ip6s;
+		for(my $i=0; $i<@{$b->{'address6'}}; $i++) {
+			push(@ip6s, $b->{'address6'}->[$i]."/".
+				    $b->{'netmask6'}->[$i]);
+			}
+		if ((@ip6s || $b->{'auto6'}) && lc($conf{'IPV6INIT'}) ne 'yes') {
+			$conf{'IPV6INIT'} = 'yes';
+			}
+		elsif (!@ip6s && !$b->{'auto6'}) {
+			$conf{'IPV6INIT'} = 'no';
+			}
+		if (@ip6s) {
+			$conf{'IPV6ADDR'} = shift(@ip6s);
+			$conf{'IPV6ADDR_SECONDARIES'} = join(" ", @ip6s);
+			}
+		if ($b->{'fullname'} =~ /^br(\d+)$/) {
+			&has_command("brctl") ||
+				&error("Bridges cannot be created unless the brctl ".
+				       "command is installed");
+			$conf{'TYPE'} = 'Bridge';
+			}
+		if ($b->{'fullname'} =~ /^bond(\d+)$/) {
+			$conf{'BONDING_OPTS'} = "mode=$b->{'mode'}";
+			if ($b->{'miimon'}) {
+				$conf{'BONDING_OPTS'} .= " miimon=$b->{'miimon'}";
+				}
+			if ($b->{'updelay'}) {
+				$conf{'BONDING_OPTS'} .= " updelay=$b->{'updelay'}";
+				}
+			if ($b->{'downdelay'}) {
+				$conf{'BONDING_OPTS'} .= " downdelay=$b->{'downdelay'}";
+				}
 
-		my @values = split(/\s+/, $b->{'partner'});
-		foreach my $val (@values) {
-			&save_bond_interface($val, $b->{'fullname'});
+			my @values = split(/\s+/, $b->{'partner'});
+			foreach my $val (@values) {
+				&save_bond_interface($val, $b->{'fullname'});
+				}
+			}
+		if ($b->{'vlan'} == 1) {
+			$conf{'VLAN'} = "yes";
 			}
 		}
-	if ($b->{'vlan'} == 1) {
-		$conf{'VLAN'} = "yes";
+	$conf{'NAME'} = $b->{'desc'};
+	if (!-r $file) {
+		# New interfaces shouldn't be controller by network manager
+		$conf{'NM_CONTROLLED'} = 'no';
 		}
-	}
-$conf{'NAME'} = $b->{'desc'};
-if (!-r $file) {
-	# New interfaces shouldn't be controller by network manager
-	$conf{'NM_CONTROLLED'} = 'no';
-	}
-&write_env_file($file, \%conf);
+	&write_env_file($file, \%conf);
 
-# If this is a bridge, set BRIDGE in real interface
-if ($b->{'bridge'}) {
-	foreach my $efile (glob("$net_scripts_dir/ifcfg-e*")) {
-		local %bconf;
-		&lock_file($efile);
-		&read_env_file($efile, \%bconf);
-		if ($bconf{'DEVICE'} eq $b->{'bridgeto'} &&
-		    $b->{'bridgeto'}) {
-			# Correct device for bridge
-			$bconf{'BRIDGE'} = $b->{'fullname'};
-			&write_env_file($efile, \%bconf);
+	# If this is a bridge, set BRIDGE in real interface
+	if ($b->{'bridge'}) {
+		foreach my $efile (glob("$net_scripts_dir/ifcfg-e*")) {
+			my %bconf;
+			&lock_file($efile);
+			&read_env_file($efile, \%bconf);
+			if ($bconf{'DEVICE'} eq $b->{'bridgeto'} &&
+			    $b->{'bridgeto'}) {
+				# Correct device for bridge
+				$bconf{'BRIDGE'} = $b->{'fullname'};
+				&write_env_file($efile, \%bconf);
+				}
+			elsif ($bconf{'BRIDGE'} eq $b->{'fullname'} &&
+			       $bconf{'BRIDGE'}) {
+				# Was using this bridge, shouldn't be
+				delete($bconf{'BRIDGE'});
+				&write_env_file($efile, \%bconf);
+				}
+			&unlock_file($efile);
 			}
-		elsif ($bconf{'BRIDGE'} eq $b->{'fullname'} &&
-		       $bconf{'BRIDGE'}) {
-			# Was using this bridge, shouldn't be
-			delete($bconf{'BRIDGE'});
-			&write_env_file($efile, \%bconf);
+		}
+
+	# Link to devices directory
+	if (-d &translate_filename($devices_dir)) {
+		&link_file($file, "$devices_dir/ifcfg-$name");
+		}
+	&unlock_file($file);
+
+	# Make sure IPv6 is enabled globally
+	if (@{$b->{'address6'}}) {
+		my %conf;
+		&lock_file($network_config);
+		&read_env_file($network_config, \%conf);
+		if (lc($conf{'NETWORKING_IPV6'}) ne 'yes') {
+			$conf{'NETWORKING_IPV6'} = 'yes';
+			&write_env_file($network_config, \%conf);
 			}
-		&unlock_file($efile);
+		&unlock_file($network_config);
 		}
 	}
-
-# Link to devices directory
-if (-d &translate_filename($devices_dir)) {
-	&link_file($file, "$devices_dir/ifcfg-$name");
-	}
-&unlock_file($file);
-
-# Make sure IPv6 is enabled globally
-if (@{$b->{'address6'}}) {
-	local %conf;
-	&lock_file($network_config);
-	&read_env_file($network_config, \%conf);
-	if (lc($conf{'NETWORKING_IPV6'}) ne 'yes') {
-		$conf{'NETWORKING_IPV6'} = 'yes';
-		&write_env_file($network_config, \%conf);
-		}
-	&unlock_file($network_config);
-	}
 }
 
 # delete_interface(&details)
@@ -367,17 +401,37 @@ if (@{$b->{'address6'}}) {
 sub delete_interface
 {
 my ($b) = @_;
-local $name = $b->{'range'} ne "" ? $b->{'name'}."-range".
-				    $b->{'range'} :
-	      $b->{'virtual'} ne "" ? $b->{'name'}.":".$b->{'virtual'}
-				    : $b->{'name'};
 my $file = $b->{'file'} || "$net_scripts_dir/ifcfg-$name";
-&lock_file($file);
-&unlink_file("$net_scripts_dir/ifcfg-$name");
-if (-d &translate_filename($devices_dir)) {
-	&unlink_file("$devices_dir/ifcfg-$name");
+if ($b->{'virtual'} ne '' && $b->{'parent'}) {
+	# Virtual interface in it's parent's file .. just remove the IP
+	my %conf;
+	my $i = $b->{'virtual'}+1;
+	&lock_file($file);
+	&read_env_file($file, \%conf);
+	for($n=$i; defined($conf{'IPADDR'.$n}); $n++) {
+		$conf{'IPADDR'.$n} = $conf{'IPADDR'.($n+1)};
+		$conf{'NETMASK'.$n} = $conf{'NETMASK'.($n+1)};
+		$conf{'PREFIX'.$n} = $conf{'PREFIX'.($n+1)};
+		delete($conf{'IPADDR'.($n+1)});
+		delete($conf{'NETMASK'.($n+1)});
+		delete($conf{'PREFIX'.($n+1)});
+		}
+	&write_env_file($file, \%conf);
+	&unlock_file($file);
+	}
+else {
+	# Interface has it's own file that should be deleted
+	my $name = $b->{'range'} ne "" ? $b->{'name'}."-range".
+					 $b->{'range'} :
+		   $b->{'virtual'} ne "" ? $b->{'name'}.":".$b->{'virtual'}
+					 : $b->{'name'};
+	&lock_file($file);
+	&unlink_file("$net_scripts_dir/ifcfg-$name");
+	if (-d &translate_filename($devices_dir)) {
+		&unlink_file("$devices_dir/ifcfg-$name");
+		}
+	&unlock_file($file);
 	}
-&unlock_file($file);
 }
 
 # can_edit(what)
@@ -392,6 +446,11 @@ else {
 	}
 }
 
+sub can_broadcast_def
+{
+return 1;
+}
+
 # valid_boot_address(address)
 # Is some address valid for a bootup interface
 sub valid_boot_address
@@ -402,20 +461,20 @@ return &check_ipaddress($_[0]);
 # get_hostname()
 sub get_hostname
 {
-local %conf;
+my %conf;
 &read_env_file($network_config, \%conf);
 if ($conf{'HOSTNAME'}) {
 	return $conf{'HOSTNAME'};
 	}
-return &get_system_hostname(1);
+return &get_system_hostname();
 }
 
 # save_hostname(name)
 sub save_hostname
 {
-local $old = &get_hostname();
-local %conf;
-&system_logged("hostname $_[0] >/dev/null 2>&1");
+my $old = &get_hostname();
+my %conf;
+&system_logged("hostname ".quotemeta($_[0])." >/dev/null 2>&1");
 &open_lock_tempfile(HOST, ">/etc/HOSTNAME");
 &print_tempfile(HOST, $_[0],"\n");
 &close_tempfile(HOST);
@@ -427,7 +486,7 @@ $conf{'HOSTNAME'} = $_[0];
 
 # If any ifcfg-XXX files have the old hostname in DHCP_HOSTNAME, fix it
 foreach my $b (&boot_interfaces()) {
-	local %ifc;
+	my %ifc;
 	&read_env_file($b->{'file'}, \%ifc);
 	if ($ifc{'DHCP_HOSTNAME'} eq $old) {
 		$ifc{'DHCP_HOSTNAME'} = $_[0];
@@ -444,13 +503,19 @@ if (-r "/etc/hostname") {
 	&close_tempfile(HOST);
 	}
 
+# Use the hostnamectl command as well
+if (&has_command("hostnamectl")) {
+	&system_logged("hostnamectl set-hostname ".quotemeta($_[0]).
+		       " >/dev/null 2>&1");
+	}
+
 undef(@main::get_system_hostname);	# clear cache
 }
 
 # get_domainname()
 sub get_domainname
 {
-local $d;
+my $d;
 &execute_command("domainname", undef, \$d, undef);
 chop($d);
 return $d;
@@ -459,7 +524,7 @@ return $d;
 # save_domainname(domain)
 sub save_domainname
 {
-local %conf;
+my %conf;
 &execute_command("domainname ".quotemeta($_[0]));
 &read_env_file($network_config, \%conf);
 if ($_[0]) {
@@ -473,7 +538,7 @@ else {
 
 sub routing_config_files
 {
-local @rv = ( $network_config, $sysctl_config );
+my @rv = ( $network_config, $sysctl_config );
 if (!$supports_dev_routes) {
 	push(@rv, $static_route_config);
 	}
@@ -499,7 +564,7 @@ return ( "/etc/HOSTNAME", $network_config );
 
 sub routing_input
 {
-local (%conf, @st, @hr, %sysctl);
+my (%conf, @st, @hr, %sysctl);
 &read_env_file($network_config, \%conf);
 if (!$supports_dev_gateway) {
 	# show default router and device
@@ -514,7 +579,7 @@ if (!$supports_dev_gateway) {
 else {
 	# multiple default routers can exist, one per interface
 	my @table;
-	local $r = 0;
+	my $r = 0;
 	if ($conf{'GATEWAY'} || $conf{'IPV6_DEFAULTGW'}) {
 		push(@table, [
 		    &interface_sel("gatewaydev$r",
@@ -525,7 +590,7 @@ else {
 		    ]);
 		$r++;
 		}
-	local @boot = &boot_interfaces();
+	my @boot = &boot_interfaces();
 	foreach $b (grep { $_->{'gateway'} && $_->{'virtual'} eq '' } @boot) {
 		push(@table, [ &interface_sel("gatewaydev$r", $b->{'name'}),
 			       &ui_textbox("gateway$r", $b->{'gateway'}, 15),
@@ -577,13 +642,13 @@ if (!$supports_dev_routes) {
 	}
 else {
 	# get static routes from per-interface files
-	local $f;
+	my $f;
 	opendir(DIR, &translate_filename($route_files_dir));
 	while($f = readdir(DIR)) {
 		if ($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
 		    $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) {
-			local $dev = $1;
-			local (%rfile, $i);
+			my $dev = $1;
+			my (%rfile, $i);
 			&read_env_file("$route_files_dir/$f", \%rfile);
 			for($i=0; defined($rfile{"ADDRESS$i"}); $i++) {
 				if ($rfile{"GATEWAY$i"}) {
@@ -605,7 +670,7 @@ else {
 # Show static network routes
 my @table;
 for($i=0; $i<=@st; $i++) {
-	local $st = $st[$i];
+	my $st = $st[$i];
 	push(@table, [ &ui_textbox("dev_$i", $st->[0], 6),
 		       &ui_textbox("net_$i", $st->[1], 15),
 		       &ui_textbox("netmask_$i", $st->[2], 15),
@@ -619,7 +684,7 @@ print &ui_table_row($text{'routes_static'},
 # Show static host routes
 my @table;
 for($i=0; $i<=@hr; $i++) {
-	local $st = $hr[$i];
+	my $st = $hr[$i];
 	push(@table, [ &ui_textbox("ldev_$i", $st->[0], 6),
 		       &ui_textbox("lnet_$i", $st->[1], 15),
 		       &ui_textbox("lnetmask_$i", $st->[2], 15) ]);
@@ -632,26 +697,26 @@ print &ui_table_row($text{'routes_local'},
 
 sub parse_routing
 {
-local (%conf, @st, %sysctl, %st, @boot);
+my (%conf, @st, %sysctl, %st, @boot);
 &lock_file($network_config);
 &read_env_file($network_config, \%conf);
 if (!$supports_dev_gateway) {
 	# Just update a single file
 	if ($in{'gateway_def'}) { delete($conf{'GATEWAY'}); }
 	elsif (!&to_ipaddress($in{'gateway'})) {
-		&error(&text('routes_edefault', $in{'gateway'}));
+		&error(&text('routes_edefault', &html_escape($in{'gateway'})));
 		}
 	else { $conf{'GATEWAY'} = $in{'gateway'}; }
 
 	if ($in{'gatewaydev_def'}) { delete($conf{'GATEWAYDEV'}); }
 	elsif ($in{'gatewaydev'} !~ /^\S+$/) {
-		&error(&text('routes_edevice', $in{'gatewaydev'}));
+		&error(&text('routes_edevice', &html_escape($in{'gatewaydev'})));
 		}
 	else { $conf{'GATEWAYDEV'} = $in{'gatewaydev'}; }
 	}
 else {
 	# Multiple defaults can be specified!
-	local ($r, $b);
+	my ($r, $b);
 	@boot = grep { $->{'virtual'} eq '' } &boot_interfaces();
 	foreach $b (@boot) {
 		delete($b->{'gateway'});
@@ -675,10 +740,10 @@ else {
 			}
 		else {
 			# For a specific interface
-			local ($b) = grep { $_->{'fullname'} eq
+			my ($b) = grep { $_->{'fullname'} eq
 					    $in{"gatewaydev$r"} } @boot;
 			$b->{'gateway'} && &error(&text('routes_eclash2',
-							$in{"gatewaydev$r"}));
+							&html_escape($in{"gatewaydev$r"})));
 			$b->{'gateway'} = $in{"gateway$r"};
 			$b->{'gateway6'} = $in{"gateway6$r"};
 			}
@@ -695,14 +760,14 @@ else {
 	$sysctl{'net.ipv4.ip_forward'} = $in{'forward'};
 	}
 
-# Parse static and local routes
+# Parse static and my routes
 for($i=0; defined($dev = $in{"dev_$i"}); $i++) {
 	next if (!$dev);
 	$net = $in{"net_$i"}; $netmask = $in{"netmask_$i"}; $gw = $in{"gw_$i"};
-	$dev =~ /^\S+$/ || &error(&text('routes_edevice', $dev));
-	&to_ipaddress($net) || &error(&text('routes_enet', $net));
-	&check_ipaddress($netmask) || &error(&text('routes_emask', $netmask));
-	&to_ipaddress($gw) || &error(&text('routes_egateway', $gw));
+	$dev =~ /^\S+$/ || &error(&text('routes_edevice', &html_escape($dev)));
+	&to_ipaddress($net) || &error(&text('routes_enet', &html_escape($net)));
+	&check_ipaddress($netmask) || &error(&text('routes_emask', &html_escape($netmask)));
+	&to_ipaddress($gw) || &error(&text('routes_egateway', &html_escape($gw)));
 	if ($netmask eq "255.255.255.255") {
 		push(@st, "$dev host $net gw $gw\n");
 		}
@@ -714,11 +779,11 @@ for($i=0; defined($dev = $in{"dev_$i"}); $i++) {
 for($i=0; defined($dev = $in{"ldev_$i"}); $i++) {
 	$net = $in{"lnet_$i"}; $netmask = $in{"lnetmask_$i"};
 	next if (!$dev && !$net);
-	$dev =~ /^\S+$/ || &error(&text('routes_edevice', $dev));
+	$dev =~ /^\S+$/ || &error(&text('routes_edevice', &html_escape($dev)));
 	&to_ipaddress($net) ||
 	    $net =~ /^(\S+)\/(\d+)$/ && &to_ipaddress("$1") ||
-		&error(&text('routes_enet', $net));
-	&check_ipaddress($netmask) || &error(&text('routes_emask', $netmask));
+		&error(&text('routes_enet', &html_escape($net)));
+	&check_ipaddress($netmask) || &error(&text('routes_emask', &html_escape($netmask)));
 	if ($netmask eq "255.255.255.255") {
 		push(@st, "$dev host $net\n");
 		}
@@ -735,7 +800,7 @@ if (!$supports_dev_routes) {
 	}
 else {
 	# Write to one file per interface (delete old, then save new/updated)
-	local $f;
+	my $f;
 	opendir(DIR, &translate_filename($route_files_dir));
 	while($f = readdir(DIR)) {
 		if (($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
@@ -747,7 +812,7 @@ else {
 	closedir(DIR);
 	foreach $dev (keys %st) {
 		$f = $supports_route_dev ? "route-$dev" : "$dev.route";
-		local (%rfile, $i);
+		my (%rfile, $i);
 		for($i=0; $i<@{$st{$dev}}; $i++) {
 			$rfile{"ADDRESS$i"} = $st{$dev}->[$i]->[0];
 			$rfile{"NETMASK$i"} = $st{$dev}->[$i]->[1];
@@ -771,29 +836,19 @@ if (%sysctl) {
 	&unlock_file($sysctl_config);
 	}
 if (@boot) {
-	local $b;
+	my $b;
 	foreach $b (@boot) {
 		&save_interface($b);
 		}
 	}
 }
 
-sub os_feedback_files
-{
-opendir(DIR, $net_scripts_dir);
-local @f = readdir(DIR);
-closedir(DIR);
-return ( (map { "$net_scripts_dir/$_" } grep { /^ifcfg-/ } @f),
-	 $network_config, $static_route_config, "/etc/resolv.conf",
-	 "/etc/nsswitch.conf", "/etc/HOSTNAME" );
-}
-
 # interface_sel(name, value)
 # Returns a menu for all boot-time interfaces
 sub interface_sel
 {
-local ($name, $value) = @_;
-local @opts = ( [ "", "&nbsp;" ],
+my ($name, $value) = @_;
+my @opts = ( [ "", "&nbsp;" ],
 		[ "*", $text{'routes_any'} ] );
 @boot_interfaces_cache = sort { $a->{'fullname'} cmp $b->{'fullname'} }
 	&boot_interfaces() if (!@boot_interfaces_cache);
@@ -807,11 +862,17 @@ return &ui_select($name, $value, \@opts);
 # Apply the interface and routing settings
 sub apply_network
 {
-if ($gconfig{'os_type'} eq 'mandrake-linux') {
-	&system_logged("(cd / ; service network stop ; service network start) >/dev/null 2>&1");
+&foreign_require("init");
+if (&init::action_status("network")) {
+	# Older system that has an init.d/network script
+	my $err = &init::restart_action("network");
+	&error($err) if ($err);
 	}
 else {
-	&system_logged("(cd / ; /etc/init.d/network stop ; /etc/init.d/network start) >/dev/null 2>&1");
+	# Probably CentOS 8 that has no apply command
+	foreach my $b (&boot_interfaces()) {
+		&apply_interface($b) if ($b->{'virtual'} eq '');
+		}
 	}
 }
 
@@ -819,7 +880,21 @@ else {
 # Calls an OS-specific function to make a boot-time interface active
 sub apply_interface
 {
-local $out = &backquote_logged("cd / ; ifdown '$_[0]->{'fullname'}' >/dev/null 2>&1 </dev/null ; ifup '$_[0]->{'fullname'}' 2>&1 </dev/null");
+my ($i) = @_;
+my $out;
+if ($i->{'parent'} && $i->{'virtual'} ne '') {
+	$out = &backquote_logged(
+		"cd / ; ".
+		"ifup ".quotemeta($i->{'name'})." 2>&1 </dev/null");
+	}
+else {
+	$out = &backquote_logged(
+		"cd / ; ".
+		"ifdown ".quotemeta($i->{'fullname'}).
+			" >/dev/null 2>&1 </dev/null ; ".
+		"ifup ".quotemeta($i->{'fullname'}).
+			" 2>&1 </dev/null");
+	}
 return $? || $out =~ /error/i ? $out : undef;
 }
 
@@ -827,7 +902,9 @@ return $? || $out =~ /error/i ? $out : undef;
 # Calls an OS-specific function to make a boot-time interface inactive
 #sub unapply_interface
 #{
-#local $out = &backquote_logged("cd / ; ifdown '$_[0]->{'fullname'}' 2>&1 </dev/null");
+#my ($i) = @_;
+#my $out = &backquote_logged(
+#	"cd / ; ifdown ".quotemeta($i->{'fullname'})." 2>&1 </dev/null");
 #return $? ? $out : undef;
 #}
 
@@ -836,10 +913,10 @@ return $? || $out =~ /error/i ? $out : undef;
 # settings.
 sub get_default_gateway
 {
-local %conf;
+my %conf;
 &read_env_file($network_config, \%conf);
-local @boot = &boot_interfaces();
-local ($gifc) = grep { $_->{'gateway'} && $_->{'virtual'} eq '' } @boot;
+my @boot = &boot_interfaces();
+my ($gifc) = grep { $_->{'gateway'} && $_->{'virtual'} eq '' } @boot;
 return ( $gifc->{'gateway'}, $gifc->{'fullname'} ) if ($gifc);
 return $conf{'GATEWAY'} ? ( $conf{'GATEWAY'}, $conf{'GATEWAYDEV'} ) : ( );
 }
@@ -849,14 +926,15 @@ return $conf{'GATEWAY'} ? ( $conf{'GATEWAY'}, $conf{'GATEWAYDEV'} ) : ( );
 # in the boot time settings.
 sub set_default_gateway
 {
+my ($gw, $gwdev) = @_;
+my %conf;
 &lock_file($network_config);
 &read_env_file($network_config, \%conf);
 if (!$supports_dev_gateway) {
 	# Just update the network config file
-	local %conf;
-	if ($_[0]) {
-		$conf{'GATEWAY'} = $_[0];
-		$conf{'GATEWAYDEV'} = $_[1];
+	if ($gw) {
+		$conf{'GATEWAY'} = $gw;
+		$conf{'GATEWAYDEV'} = $gwdev;
 		}
 	else {
 		delete($conf{'GATEWAY'});
@@ -865,11 +943,11 @@ if (!$supports_dev_gateway) {
 	}
 else {
 	# Set the gateway in the specified interface file, and clear the rest
-	local @boot = grep { $->{'virtual'} eq '' } &boot_interfaces();
+	my @boot = grep { $->{'virtual'} eq '' } &boot_interfaces();
 	foreach $b (@boot) {
 		delete($b->{'gateway'});
-		if ($_[0] && $b->{'fullname'} eq $_[1]) {
-			$b->{'gateway'} = $_[0];
+		if ($gw && $b->{'fullname'} eq $gwdev) {
+			$b->{'gateway'} = $gw;
 			&save_interface($b);
 			}
 		}
@@ -885,10 +963,10 @@ else {
 # boot time settings.
 sub get_default_ipv6_gateway
 {
-local %conf;
+my %conf;
 &read_env_file($network_config, \%conf);
-local @boot = &boot_interfaces();
-local ($gifc) = grep { $_->{'gateway6'} && $_->{'virtual'} eq '' } @boot;
+my @boot = &boot_interfaces();
+my ($gifc) = grep { $_->{'gateway6'} && $_->{'virtual'} eq '' } @boot;
 return ( $gifc->{'gateway6'}, $gifc->{'fullname'} ) if ($gifc);
 return $conf{'IPV6_DEFAULTGW'} ? ( $conf{'IPV6_DEFAULTGW'},
 				   $conf{'IPV6_DEFAULTDEV'} ) : ( );
@@ -901,7 +979,7 @@ sub set_default_ipv6_gateway
 {
 &lock_file($network_config);
 &read_env_file($network_config, \%conf);
-local @boot = grep { $->{'virtual'} eq '' } &boot_interfaces();
+my @boot = grep { $->{'virtual'} eq '' } &boot_interfaces();
 foreach $b (@boot) {
 	delete($b->{'gateway6'});
 	if ($_[0] && $b->{'fullname'} eq $_[1]) {
@@ -946,7 +1024,7 @@ return $gconfig{'os_type'} eq 'redhat-linux' &&
 # Print HTML for a IP range interface
 sub range_input
 {
-local $new = !$_[0];
+my $new = !$_[0];
 
 # Range description
 print &ui_table_row($text{'ifcs_desc'},
@@ -985,7 +1063,7 @@ print &ui_table_row($text{'range_num'},
 # parse_range(&range, &in)
 sub parse_range
 {
-local %in = %{$_[1]};
+my %in = %{$_[1]};
 if ($in{'new'}) {
 	$_[0]->{'name'} = $in{'iface'};
 	$in{'range'} =~ /^[a-z0-9\.\_]+$/ || &error($text{'range_ename'});
@@ -1000,8 +1078,8 @@ $_[0]->{'start'} = $in{'start'};
 &check_ipaddress($in{'end'}) || &error($text{'range_eend'});
 $_[0]->{'end'} = $in{'end'};
 
-local @sip = split(/\./, $in{'start'});
-local @eip = split(/\./, $in{'end'});
+my @sip = split(/\./, $in{'start'});
+my @eip = split(/\./, $in{'end'});
 $sip[0] == $eip[0] && $sip[1] == $eip[1] && $sip[2] == $eip[2] ||
 	&error($text{'range_eclass'});
 $sip[3] <= $eip[3] || &error($text{'range_ebefore'});
@@ -1017,10 +1095,10 @@ sub get_dhcp_hostname
 {
 return -1 if ($gconfig{'os_type'} ne 'redhat-linux' ||
 	      $gconfig{'os_version'} < 11);
-local @boot = &boot_interfaces();
-local ($eth) = grep { $_->{'fullname'} =~ /^(eth|em)\d+$/ } @boot;
+my @boot = &boot_interfaces();
+my ($eth) = grep { $_->{'fullname'} =~ /^(eth|em)\d+$/ } @boot;
 return -1 if (!$eth);
-local %eth;
+my %eth;
 &read_env_file($eth->{'file'}, \%eth);
 return $eth{'DHCP_HOSTNAME'} ne &get_system_hostname();
 }
@@ -1036,10 +1114,10 @@ sub save_dhcp_hostname
 # Gets the teamingpartners of a configured bond interface
 sub get_teaming_partner
 {
-local ($g, $return);
+my ($g, $return);
 opendir(CONF2, &translate_filename($net_scripts_dir));
 while($g = readdir(CONF2)) {
-        local %conf2;
+        my %conf2;
         if ($g !~ /\.(bak|old)$/i && $g =~ /^ifcfg-([a-z0-9:\.]+)$/) {
                 &read_env_file("$net_scripts_dir/$g", \%conf2);
                 if ($conf2{'MASTER'} eq "$_[0]") {
@@ -1059,7 +1137,7 @@ return $_[0] =~ /^(eth|em)/;
 # Returns 1 if managing IPv6 interfaces is supported
 sub supports_address6
 {
-local ($iface) = @_;
+my ($iface) = @_;
 return !$iface || $iface->{'virtual'} eq '';
 }
 
@@ -1076,21 +1154,31 @@ sub supports_bridges
 return 1;
 }
 
+# supports_dns_interface(&iface)
+# Returns 1 if DNS servers can be set in interface config files
+sub supports_dns_interface
+{
+my ($ifc) = @_;
+return $ifc->{'DNS1'} || $gconfig{'os_version'} >= 15 &&
+		         &iface_type($ifc->{'DEVICE'}) =~ /Ethernet/i;
+}
+
 # os_save_dns_config(&config)
 # Updates DNSx lines in all network-scripts files that have them
 sub os_save_dns_config
 {
-local ($conf) = @_;
-foreach my $b (&boot_interfaces()) {
-	local %ifc;
+my ($conf) = @_;
+my @boot = &boot_interfaces();
+foreach my $b (@boot) {
+	my %ifc;
 	&read_env_file($b->{'file'}, \%ifc);
-	next if (!defined($ifc{'DNS1'}));
+	next if (!&supports_dns_interface(\%ifc));
 	&lock_file($b->{'file'});
 	foreach my $k (keys %ifc) {
 		delete($ifc{$k}) if ($k =~ /^DNS\d+$/);
 		}
 	&write_env_file($b->{'file'}, \%ifc);
-	local $i = 1;
+	my $i = 1;
 	foreach my $ns (@{$conf->{'nameserver'}}) {
 		$ifc{'DNS'.$i} = $ns;
 		$i++;
@@ -1100,6 +1188,13 @@ foreach my $b (&boot_interfaces()) {
 		# later if DNS resolves come back
 		$ifc{'DNS1'} = ''
 		}
+	my @d = @{$conf->{'domain'}};
+	if (@d) {
+		$ifc{'DOMAIN'} = $d[0];
+		}
+	else {
+		delete($ifc{'DOMAIN'});
+		}
 	&write_env_file($b->{'file'}, \%ifc);
 	&unlock_file($b->{'file'});
 	}
